// file: $USFS/class/image_segmentation/v1.0/imags_proc_1.cc
//

// isip include files
//
#include "image_segmentation.h"
#include "image_segmentation_constants.h"

// other include files
//
#ifndef __ISIP_PPM
#include <ppm.h>
#endif

// method: train_cc
//
// arguments:
//   char_1* image_file: (input) the image file name
//   char_1* seg_file: (input) the segmentation data file name
//   
// return:
//   a logical_1 value indicating status
//
// this method trains the model on the given image file
//
logical_1 Image_segmentation::train_cc(char_1* image_file_a,
				       char_1* seg_file_a) {
  // declare local variables
  //
  Ppm ppm;

  // a set of flags for initializing the image_analysis objects
  //
  static logical_1 ia_init[IMAGS_NUM_MODELS] = {
    ISIP_TRUE, ISIP_TRUE, ISIP_TRUE, ISIP_TRUE, ISIP_TRUE, ISIP_TRUE
  };

  // check if the parameters are valid
  //
  if (image_file_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"train_cc", (char_1*)"no image file
                     specified");
    return ISIP_FALSE;
  }

  if (seg_file_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"train_cc", (char_1*)"no segmentation
                     file specified");
    return ISIP_FALSE;
  }
  
  // open image file
  //
  if (ppm.open_cc(image_file_a, ISIP_READ_ONLY) == ISIP_FALSE) {
    error_handler_cc((char_1*)"train_cc", (char_1*)"error opening
                     image file");
    return ISIP_FALSE;
  }

  // read the dimensions of the image
  //
  int_4 width = ppm.get_width_cc();
  int_4 height = ppm.get_height_cc();

  // prepare for training
  //
  for (int_4 m=0; m<num_models_d; m++) {

    // load segmentation data
    //
    if (ia_d[m].load_polygons_cc(seg_file_a) == ISIP_FALSE) {
      error_handler_cc((char_1*)"train_cc", (char_1*)"error loading
                       segmentation data");
      return ISIP_FALSE;
    }
  }

  // check validity of the size data
  //
  if (win_size_d < frm_size_d) {
    error_handler_cc((char_1*)"train_cc", (char_1*)"window size is less than
                     frame size");
    return ISIP_FALSE;
  }
  if ((dct_blk_size_d > 0) && (dct_blk_size_d < frm_size_d)) {
    error_handler_cc((char_1*)"train_cc", (char_1*)"dct block size is less
                     than frame size");
    return ISIP_FALSE;
  }
  
  // choose the larger size as the block size used to load buffer
  // from image file
  //
  int_4 blk_size = win_size_d;
  if (blk_size < dct_blk_size_d) {
    blk_size = dct_blk_size_d;
  }

  // loop over the image file
  //
  for (int_4 i=0; i<height; i+=frm_size_d) {
    for (int_4 j=0; j<width; j+=frm_size_d) {
      
      // parameters related to frame/window, note the block size
      // we use to load data is not necessarily win_size_d
      //
      int_4 frm_row_npix = frm_size_d;
      int_4 frm_col_npix = frm_size_d;
      int_4 win_row_npix = blk_size;
      int_4 win_col_npix = blk_size;
      int_4 dif_frm_win = (blk_size - frm_size_d)/2;
      int_4 frm_row_num = j;
      int_4 frm_col_num = i;
      int_4 win_row_num = 0;
      int_4 win_col_num = 0;
      
      // checking for the out of bounds data
      //
      if ((j + frm_size_d) >= width) {
        frm_row_npix = width - j;
        win_row_npix = blk_size - (frm_size_d - frm_row_npix);
      }
      
      if ((i + frm_size_d) >= height) {
        frm_col_npix = height - i;
        win_col_npix = blk_size - (frm_size_d - frm_col_npix);
      }

      // a flag indicating if zero padding is necessary
      //
      logical_1 zero_padding = ISIP_FALSE;

      // declare buffers for reading the ppm image in blocks
      //
      uint_1* win_red = new uint_1[win_row_npix * win_col_npix];   
      uint_1* win_green = new uint_1[win_row_npix * win_col_npix];
      uint_1* win_blue = new uint_1[win_row_npix * win_col_npix];
    
      // parameters used for reading a block from the image
      //
      int_4 actual_off_w = 0;
      int_4 actual_off_h = 0;
      int_4 actual_width = win_row_npix;
      int_4 actual_height = win_col_npix;

      // compute the actual block position and size need to be read
      // from the image
      //
      if ((win_row_num = (frm_row_num - dif_frm_win)) < 0) {
        actual_off_w = 0;
        actual_width = actual_width - (actual_off_w - win_row_num);
        zero_padding = ISIP_TRUE;
      }
      else {
        actual_off_w = win_row_num;
      }

      if ((win_row_num + win_row_npix) > width) {
        actual_width = width - actual_off_w;
        zero_padding = ISIP_TRUE;
      }
	
      if ((win_col_num = (frm_col_num - dif_frm_win)) < 0) {
        actual_off_h = 0;
        actual_height = actual_height - (actual_off_h - win_col_num);
        zero_padding = ISIP_TRUE;
      }
      else {
        actual_off_h = win_col_num;
      }

      if ((win_col_num + win_col_npix) > height) {
        actual_height = height - actual_off_h;
        zero_padding = ISIP_TRUE;
      }
      
      // read the ppm file block by block
      //
      if (ppm.read_cc(win_red, win_green, win_blue, actual_off_w,
		      actual_off_h, actual_width, actual_height)
	  == ISIP_FALSE) {
	error_handler_cc((char_1*)"train_cc",
       		         (char_1*)"error reading the image file");
	return ISIP_FALSE;
      }

      // do zero padding if necessary
      //
      if (zero_padding == ISIP_TRUE) {
        if (pad_zero_cc(win_red, win_green, win_blue, win_row_num,
			win_col_num, win_row_npix, win_col_npix,
			actual_off_w, actual_off_h, actual_width,
			actual_height) == ISIP_FALSE) {
	  error_handler_cc((char_1*)"train_cc", (char_1*)"error
                           doing zero padding");
	  return ISIP_FALSE;
	}
      }
	
      // train the models
      //
      for (int_4 m=0; m<num_models_d; m++) {

	// initialize
        //
	if (ia_init[m] == ISIP_TRUE) {
          if (ia_d[m].init_cc() == ISIP_FALSE) {
            error_handler_cc((char_1*)"train_cc", (char_1*)"error initilizing
                             the Image_analysis objects");
            return ISIP_FALSE;
          }
	  ia_init[m] = ISIP_FALSE;
	}

	if (ia_d[m].train_cc(win_red, win_green, win_blue, win_row_num,
			     win_col_num, win_row_npix, win_col_npix)
	    == ISIP_FALSE) {
	  error_handler_cc((char_1*)"train_cc",
			 (char_1*)"error training the models");
	  return ISIP_FALSE;
        }
      }

      // release memory
      //
      if (win_red != (uint_1*)NULL) {
        delete [] win_red;
      }

      if (win_green != (uint_1*)NULL) {
        delete [] win_green;
      }

      if (win_blue != (uint_1*)NULL) {
        delete [] win_blue;
      }
    }
  }    

  // close the image file
  //
  ppm.close_cc();
       
  // exit gracefully
  //
  return ISIP_TRUE;
}
